@@ -63,6 +63,7 @@ static uint     opt_mysql_port= 0, opt_protocol= 0;
 static char * opt_mysql_unix_port=0;
 static char *opt_plugin_dir= 0, *opt_default_auth= 0;
 static longlong opt_ignore_lines= -1;
+
 #include <sslopt-vars.h>
 
 static char **argv_to_free;
@@ -143,7 +144,7 @@ static struct my_option my_long_options[] =
   {"password", 'p',
    "Password to use when connecting to server. If password is not given it's asked from the tty.",
    0, 0, 0, GET_STR, OPT_ARG, 0, 0, 0, 0, 0, 0},
-#ifdef __WIN__
+#ifdef _WIN32
   {"pipe", 'W', "Use named pipes to connect to server.", 0, 0, 0, GET_NO_ARG,
    NO_ARG, 0, 0, 0, 0, 0, 0},
 #endif
@@ -222,7 +223,7 @@ file. The SQL command 'LOAD DATA INFILE' is used to import the rows.\n");
 
 static my_bool
 get_one_option(const struct my_option *opt, const char *argument,
-               const char *filename __attribute__((unused)))
+               const char *filename)
 {
   switch(opt->id) {
   case 'p':
@@ -246,7 +247,7 @@ get_one_option(const struct my_option *opt, const char *argument,
     else
       tty_password= 1;
     break;
-#ifdef __WIN__
+#ifdef _WIN32
   case 'W':
     opt_protocol = MYSQL_PROTOCOL_PIPE;
     break;
@@ -259,6 +260,27 @@ get_one_option(const struct my_option *opt, const char *argument,
       exit(1);
     }
     break;
+  case 'P':
+    if (filename[0] == '\0')
+    {
+      /* Port given on command line, switch protocol to use TCP */
+      opt_protocol= MYSQL_PROTOCOL_TCP;
+    }
+    break;
+  case 'S':
+    if (filename[0] == '\0')
+    {
+      /*
+        Socket given on command line, switch protocol to use SOCKETSt
+        Except on Windows if 'protocol= pipe' has been provided in
+        the config file or command line.
+      */
+      if (opt_protocol != MYSQL_PROTOCOL_PIPE)
+      {
+        opt_protocol= MYSQL_PROTOCOL_SOCKET;
+      }
+    }
+    break;
   case '#':
     DBUG_PUSH(argument ? argument : "d:t:o");
     debug_check_flag= 1;
@@ -631,7 +653,6 @@ error:
   pthread_cond_signal(&count_threshhold);
   pthread_mutex_unlock(&counter_mutex);
   mysql_thread_end();
-  pthread_exit(0);
   return 0;
 }
 
@@ -642,6 +663,9 @@ int main(int argc, char **argv)
   MY_INIT(argv[0]);
   sf_leaking_memory=1; /* don't report memory leaks on early exits */
 
+  /* We need to know if protocol-related options originate from CLI args */
+  my_defaults_mark_files = TRUE;
+
   load_defaults_or_exit("my", load_default_groups, &argc, &argv);
   /* argv is changed in the program */
   argv_to_free= argv;
@@ -650,6 +674,7 @@ int main(int argc, char **argv)
     free_defaults(argv_to_free);
     return(1);
   }
+
   sf_leaking_memory=0; /* from now on we cleanup properly */
 
   if (opt_use_threads && !lock_tables)
